#include <stdio.h>
#include <stdlib.h>

#include <mpi.h>

#define WITHBARRIER 0

int main (int argc, char ** argv) {

  int n, i, j;
  double stime, etime;
  int size, rank, rlen;
  char myname[MPI_MAX_PROCESSOR_NAME]; 

  MPI_Status status;

  MPI_Init (&argc, &argv);
  MPI_Comm_size (MPI_COMM_WORLD, &size);
  MPI_Comm_rank (MPI_COMM_WORLD, &rank);
  MPI_Get_processor_name (myname, &rlen);

  printf ("Io sono %3d di %3d e giro su %s\n", rank, size, myname);


  if (argc != 2) {
    printf ("Indica la dimensione del vettore N\n");
    MPI_Finalize ();
    return EXIT_FAILURE;
  } 

  if ( (n = atoi (argv[1])) < 0) {
    printf ("N deve essere maggiore di 0\n");
    MPI_Finalize ();
    return EXIT_FAILURE;
  }

  if (size == 2) {
    double * d;

    srand (10);

    if ( (d = (double *) malloc (sizeof (double) * n * n)) == NULL) {
      printf ("Memory allocation error\n");
      return EXIT_FAILURE;
    }

    if (rank == 0) {
      for (i=0; i<n; i++)
        for (j=0; j<n; j++)
          d[(i*n) + j] = 1.0e0 + (double) (1.0e0 * n * (rand() / (RAND_MAX + 1.0e0)));   
    }

#if WITHBARRIER
    MPI_Barrier (MPI_COMM_WORLD);
#endif
      
    if (rank == 0) {
      // errore
      //MPI_Barrier (MPI_COMM_WORLD);
      // errore
      stime = MPI_Wtime ();
      MPI_Send (d, n*n, MPI_DOUBLE, 1, 1, MPI_COMM_WORLD);
      etime = MPI_Wtime ();
      printf ("Send time: %f s\n", etime - stime);
    }
    else {
      stime = MPI_Wtime ();
      MPI_Recv (d, n*n, MPI_DOUBLE, 0, 1, MPI_COMM_WORLD, 
          &status);
      etime = MPI_Wtime ();
      printf ("Recv time: %f s\n", etime - stime);
    }

#if WITHBARRIER
    MPI_Barrier (MPI_COMM_WORLD);
#endif

    free (d);
  }

  MPI_Finalize ();

  return EXIT_SUCCESS;
}
